#!/usr/bin/env python

# sbtest v0.1: investigate QSpinBox strangeness
#
# Copyright 2002 Hans-Peter Jansen <hpj@urpla.net>
#
# This program is placed under the GNU General Public License V.2

import sys
from qt import *

qEvent = {
    0: "None",
    1: "Timer",
    2: "MouseButtonPress",
    3: "MouseButtonRelease",
    4: "MouseButtonDblClick",
    5: "MouseMove",
    6: "KeyPress",
    7: "KeyRelease",
    8: "FocusIn",
    9: "FocusOut",
    10: "Enter",
    11: "Leave",
    12: "Paint",
    13: "Move",
    14: "Resize",
    15: "Create",
    16: "Destroy",
    17: "Show",
    18: "Hide",
    19: "Close",
    20: "Quit",
    21: "Reparent",
    22: "ShowMinimized",
    23: "ShowNormal",
    24: "WindowActivate",
    25: "WindowDeactivate",
    26: "ShowToParent",
    27: "HideToParent",
    28: "ShowMaximized",
    29: "ShowFullScreen",
    30: "Accel",
    31: "Wheel",
    32: "AccelAvailable",
    33: "CaptionChange",
    34: "IconChange",
    35: "ParentFontChange",
    36: "ApplicationFontChange",
    37: "ParentPaletteChange",
    38: "ApplicationPaletteChange",
    39: "PaletteChange",
    40: "Clipboard",
    42: "Speech",
    50: "SockAct",
    51: "AccelOverride",
    52: "DeferredDelete",
    60: "DragEnter",
    61: "DragMove",
    62: "DragLeave",
    63: "Drop",
    64: "DragResponse",
    70: "ChildInserted",
    71: "ChildRemoved",
    72: "LayoutHint",
    73: "ShowWindowRequest",
    80: "ActivateControl",
    81: "DeactivateControl",
    82: "ContextMenu",
    83: "IMStart",
    84: "IMCompose",
    85: "IMEnd",
    86: "Accessibility",
    87: "Tablet"
}

probmsg = """\
Problem: reimplemented QSpinBox below
don't get focus events and doesn't behave
correctly on up/down cursor keys!\
"""

class SpinBox(QSpinBox):
    def __init__(self, minValue, maxValue, step = 1, parent = None, name = None):
        self.lastval = None
        QSpinBox.__init__(self, minValue, maxValue, step, parent, name)

    def event(self, e):
        t = e.type()
        if t in qEvent.keys():
            print "SpinBox event:", qEvent[t]
        else:
            print "unknown SpinBox event:", t
        # this seems to be necessary because of races with qApp.quit()
        if QSpinBox:
            return QSpinBox.event(self, e)
        else:
            return 0

    def focusInEvent(self, e):
        self.lastval = self.value()
        print "SpinBox focusInEvent", self.lastval
        return QSpinBox.focusInEvent(self, e)

    def focusOutEvent(self, e):
        v = self.value()
        print "SpinBox focusOutEvent", v, self.lastval
        if self.lastval != v:
            emit(SIGNAL("valueChanged(int)"), (v))
        return QSpinBox.focusOutEvent(self, e)

class PushButton(QPushButton):
    def __init__(self, text = None, parent = None, name = None):
        QPushButton.__init__(self, text, parent, name)

    def event(self, e):
        t = e.type()
        if t in qEvent.keys():
            print "PushButton event:", qEvent[t]
        else:
            print "unknown PushButton event:", t
        # this seems to be necessary because of races with qApp.quit()
        if QPushButton:
            return QPushButton.event(self, e)
        else:
            return 0

    def focusInEvent(self, e):
        print "PushButton focusInEvent"
        return QPushButton.focusInEvent(self, e)

    def focusOutEvent(self, e):
        print "PushButton focusOutEvent"
        return QPushButton.focusOutEvent(self, e)

class sbTest(QWidget):
    def __init__(self, parent = None, name = None, fl = 0):
        QWidget.__init__(self, parent, name, fl)
        self.setCaption("QSpinBox test")
        self.value = 42
        vbox = QVBoxLayout(self, 4, -1, "vbox")
        qb = PushButton("Quit", self, "qb")
        vbox.addWidget(qb)
        l = QLabel(probmsg, self, "tl")
        vbox.addWidget(l)
        sb = SpinBox(1, 200, 1, self, "sb")
        sb.setValue(self.value)
        vbox.addWidget(sb)
        self.connect(sb, SIGNAL("valueChanged(int)"), self.changed)
        self.connect(qb, SIGNAL("clicked()"), self.closeEvent)
        self.vbox = vbox

    def changed(self, v):
        print "sb changed:", v
        self.value = v

    def closeEvent(self, e = None):
        print "close app"
        qApp.quit()
        #sys.exit(0)

if __name__ == "__main__":
    a = QApplication(sys.argv)

    QObject.connect(a,SIGNAL("lastWindowClosed()"),a,SLOT("quit()"))
    w = sbTest()
    a.setMainWidget(w)
    w.show()
    a.exec_loop()

